! program performing estimates of variables not in logarithm

! In the following module, can define parameters and flags

MODULE FIRST

INTEGER,PARAMETER :: LONG=SELECTED_REAL_KIND(P=15,R=200)

!       ************    declare parameter values      *****************

INTEGER,PARAMETER :: T=125         ! Number of time periods
INTEGER,PARAMETER :: P=2           ! Total number of variables
INTEGER,PARAMETER :: M=1           ! Number of full effect variables (income in paper)
INTEGER,PARAMETER :: R=1           ! Number of Half effect variables (consumption in paper)
INTEGER,PARAMETER :: Q=2           ! Number of lags
INTEGER,PARAMETER :: C=2           ! Number of cohorts

INTEGER,PARAMETER :: NDY=0                ! 1= ESTIMATING ND & Y; 0= ESTIMATING Y & ND
INTEGER,PARAMETER :: EXC_SENS=0         ! PERFORM EXCESS SENS. TEST - WORKS ONLY WITH Q=2, adds coefficients and LogL is used for LR test

INTEGER,PARAMETER :: READ_START=0        ! 1=READ INITIAL VALUES FROM FILE

INTEGER,PARAMETER :: NCONSTR=3     ! Number of exclusion constraints
								   ! Possible values are: 0, 2, 3, 4 if Q=2   (specifications shown in the paper are 0 and 3)
								   !                      0, 4, 7, 8 if Q=4

INTEGER,PARAMETER :: H=P-(M+R)     ! Number of other variables

INTEGER(4),PARAMETER :: L=2*(M+R)*C+H      ! Number of shocks including meas.err., no lags
INTEGER(4),PARAMETER :: LQ1=((M+R)*C+H)*(Q+1)+2*C*(M+R)  ! Dimension of state vector

INTEGER,PARAMETER :: N_V1=(M+R)*(M+R+1)/2        ! VARIANCE+CONTEMPORANEOUS COV

! number of variables to be read from data set=yrq, coh,cellsize + p- vars.
! p- variables are already detrended and nothing else


INTEGER,PARAMETER :: N_VAR=M+R+3
INTEGER,PARAMETER :: N_VAR_TOT=(M+R)*C+H


INTEGER(4),PARAMETER :: T_IND=31798    !total dimension in individual data




!       ************    compute other parameters      *****************     

INTEGER(4),PARAMETER :: NUM_A=M*P*(Q+1)+(R+H)*R*(Q+1)+H*H*(Q+1)-P-NCONSTR+Q*EXC_SENS
INTEGER(4),PARAMETER :: NUM_S=(M+R)*C*(C+1)/2+H
INTEGER(4),PARAMETER :: NUM_TOT=NUM_A+NUM_S

INTEGER(4),PARAMETER :: NUM_ERR=C*((M+R)*(M+R+1)/2)

INTEGER(4),PARAMETER :: PQ1=P*(Q+1), CC1=C*(C+1)/2

!       ************    TOTAL NUMBER OF MOMENTS       *****************     

INTEGER(4),PARAMETER ::                            &
&       N_MOM=(M+R)*(M+R-1)*C*C/2+(M+R)*CC1 +   &   !  COV
&         (M+R)*Q*C*C                  +        &   !  COV1
&         (M+R)*(M+R-1)*Q*C*C          +        &   !  COV2
&          H*(H+1)/2                   +        &   !  COV3
&          H*H*Q                       +        &   !  COV3_2
&          H*(M+R)*C                   +        &   !  COV4
&          2*Q*C*(M+R)*H                            !  COV5




REAL(LONG) :: CORR_0(M+R,M+R,C),XT(N_VAR_TOT,T-1)
REAL(LONG) :: CRIT2(T-1),FMAT(LQ1,LQ1),V1(N_V1),V2(N_V1)
INTEGER(4) :: MAXCELL,CONSTRAINT(P,PQ1),ICALL

END MODULE FIRST

!**********************************************************************

PROGRAM MAXLIK

USE NUMERICAL_LIBRARIES

USE FIRST
IMPLICIT NONE

! DATA MATRICES:

REAL(LONG),ALLOCATABLE ::Z(:,:),W(:,:),Y(:,:,:,:)

REAL(LONG) :: INTRT(T,H),EPS
REAL(LONG) :: YSTD((M+R),(M+R),C,T),MCREAD(C*T,M+R)
REAL(LONG) :: BBETAVAR(N_MOM,N_MOM),DET,CRIT0(T-1)
REAL(LONG) :: DU(T-1,NUM_TOT),XOLD(NUM_TOT),VAR_X(NUM_TOT,NUM_TOT),RES_FIN(3,NUM_TOT)
REAL(LONG) :: STDV(NUM_TOT),T_STAT(NUM_TOT),WL,TEMP(T-Q-1)
REAL(LONG) :: STDV2(NUM_TOT),T_STAT2(NUM_TOT),VAR_X2(NUM_TOT,NUM_TOT),GRADX(NUM_TOT)
REAL(LONG) :: TEMPF1(L,L),TEMPF2(N_VAR_TOT,N_VAR_TOT),CORR_CHECK(M+R,M+R,C)
REAL(LONG) :: EXC,EXC_STD

INTEGER(4) :: YRQ(T_IND),CELL(T_IND),COH(T_IND),LQ1LQ1,IFOUR
INTEGER(4) :: MCELL(C,T),MCTRUE(C,T,M+R)
INTEGER(4) :: I,II,III,J,JJ,JJJ,K,KK,IO,INI,ENND,ENND2



!  ********       Variables and parameters needed by E04UCF   **********

INTEGER(4),PARAMETER :: N=NUM_TOT              ! TOT N.OF PARAMETERS
REAL(LONG) :: X(N), F, XINI(N),XSCALE(N),FSCALE,RPARAM(7),XLB(N),XUB(N)
REAL(LONG) :: FACMAT(N_MOM,N_MOM),DET1,DET2,WK(N*(2*N+8)),GRAD(N)
INTEGER :: IPARAM(7),IBTYPE,IWK(N)


REAL(8) :: CPSEC,T0,T10,T1



!  ********       Variables and parameters needed by F01ABF   **********
INTEGER,PARAMETER :: IA=N_MOM+1
REAL(LONG) :: ZINV(N_MOM)


EXTERNAL OBJFUN


! ************* Input and output files *******************************

OPEN(7,FILE='new15_yndnew4lags_33ini.dat')      ! Input file if read initial values from file
OPEN (8,FILE='new15_yndnew4lags_33.dat')        ! Write estimates at each iteration

! INPUT FILES

OPEN(10,FILE='y&nd1_2o.dat',STATUS='OLD')     ! Income 1 (gross earnings) & non durables
!OPEN(10,FILE='y&nd2_2o.dat',STATUS='OLD')     ! Income 2 (gross earnings + benefits) & non durables
!OPEN(10,FILE='y&nd3_2o.dat',STATUS='OLD')     ! Income 3 (net earnings + benefits) & non durables

!OPEN(10,FILE='y&tc1_2o.dat',STATUS='OLD')     ! Income 1 (gross earnings) & total consumption
!OPEN(10,FILE='y&tc2_2o.dat',STATUS='OLD')     ! Income 2 (gross earnings + benefits) & total consumption
!OPEN(10,FILE='y&tc3_2o.dat',STATUS='OLD')     ! Income 3 (net earnings + benefits) & total consumption


OPEN(101,FILE='cellnd&y_2o.dat',STATUS='OLD')	! cellsize, is the same for all above input files


OPEN(11,FILE='new15_excsens_yndnew2lags_111.out')     ! output file



! *******************       READ DATA       ****************************

ALLOCATE(Z(T_IND,N_VAR))
ALLOCATE(W(T_IND,(M+R)))

DO I=1,T_IND
   READ(10,*) (Z(I,J),J=1,N_VAR)
END DO


DO I=1,C*T
   READ(101,*) (MCREAD(I,J),J=1,M+R)
END DO


IF(H.GT.0) THEN
	print*,"this program does not provide estimates for h greater than zero"
	go to 1010
ENDIF


YRQ=Z(:,1)
COH=Z(:,2)
CELL=Z(:,3)
W=Z(:,4:M+R+3)


!                  DEFINE CONSTRAINT MATRIX     - CONTRAINT(P,PQ1)

! This matrix has zeros where the parameter have to be estimated, ones where the 
! parameters are constrained to be zero

CONSTRAINT=0

if(q.eq.2) then
	if(nconstr.eq.0) then
		constraint=0
	elseif(nconstr.eq.2)then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
	elseif(nconstr.eq.3) then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
		CONSTRAINT(2,6)=1
	elseif(nconstr.eq.4) then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
		CONSTRAINT(2,5)=1
		CONSTRAINT(2,6)=1
	endif
elseif(q.eq.4) then
	if(nconstr.eq.0) then
		constraint=0
	elseif(nconstr.eq.4)then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
		CONSTRAINT(1,4)=1
		CONSTRAINT(1,5)=1
	elseif(nconstr.eq.7)then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
		CONSTRAINT(1,4)=1
		CONSTRAINT(1,5)=1
		constraint(2,10)=1
		constraint(2,9)=1
		constraint(2,8)=1
	elseif(nconstr.eq.8)then
		CONSTRAINT(1,2)=1
		CONSTRAINT(1,3)=1
		CONSTRAINT(1,4)=1
		CONSTRAINT(1,5)=1
		constraint(2,10)=1
		constraint(2,9)=1
		constraint(2,8)=1
		constraint(2,7)=1
	endif
endif

! START ESTIMATION

WRITE(11,*) "N_MOM: ",N_MOM
WRITE(11,*) "N PARAM ", N
WRITE(11,*) "LONG: ",LONG


!DEALLOCATE(Z)

MAXCELL=MAXVAL(CELL)

ALLOCATE(Y(M+R,MAXCELL,C,T))
Y=0.0

IO=1

DO KK=1,T
   DO K=1,C
     DO J=1,CELL(IO)
        DO I=1,M+R
           Y(I,J,K,KK)=W(IO,I)
        END DO
        IO=IO+1
      END DO
   END DO
END DO
DEALLOCATE(W)


IO=1
DO KK=1,T
   DO K=1,C
      DO J=1,CELL(IO)
         MCELL(K,KK)=CELL(IO)
         IO=IO+1
      END DO
   END DO
END DO

IO=1
DO KK=1,T
   DO K=1,C
        MCTRUE(K,KK,:)=MCREAD(IO,:)
        IO=IO+1
   END DO
END DO

!write(11,*) y

CALL COLLAPSE(YRQ,MCELL,MCTRUE,COH,Y,INTRT,XT,CORR_0)
write(11,*) "corr0"
write(11,*) corr_0

write(11,*) " "
write(11,*) "corr_check"
II=1
DO I=1,M+R
   DO J=I,M+R
      CORR_CHECK(I,J,1)=V1(II)-2*CORR_0(I,J,1)
      CORR_CHECK(I,J,2)=V2(II)-2*CORR_0(I,J,2)  
	  II=II+1
	  WRITE(11,*) CORR_CHECK(I,J,1),CORR_CHECK(I,J,2)
   END DO
END DO

write(11,*) " "
write(11,*) "original moments"

II=1
DO I=1,M+R
   DO J=I,M+R
      WRITE(11,*) V1(II),V2(II)
      II=II+1
   END DO
END DO

DO I=1,M+R
   IF(CORR_CHECK(I,I,1).LT.0.0D0) THEN 
      PRINT*,"NEGATIVE VARIANCE: COH: 1"
	  PRINT*,I
   ENDIF

   IF(CORR_CHECK(I,I,2).LT.0.0D0) THEN 
      PRINT*,"NEGATIVE VARIANCE: COH: 2"
	  PRINT*,I
   ENDIF
END DO



1111 FORMAT(3E16.6/3E16.6/3E16.6/3E16.6/3E16.6)



DO I=1,NUM_A                                ! INITIAL VALUES
 X(I)=0.0D0
END DO
DO I=NUM_A+1,NUM_TOT
  X(I)=0.5
END DO

IF(READ_START.EQ.1) THEN
DO I=1,NUM_TOT
   READ(7,*) X(I)
END DO
ENDIF

!  F MATRIX (STATE EQ)

TEMPF1=0.0D0
TEMPF2=0.0D0
FMAT=0.0D0

DO I=1,L
   TEMPF1(I,I)=1.0D0
END DO

DO I=1,N_VAR_TOT
   TEMPF2(I,I)=1.0D0
END DO

DO I=1,Q+1
   DO J=1,Q+1
      IF((I-1.EQ.J).AND.(I.LE.2)) THEN
        ENND=I*L
	    ENND2=J*L	  
	    FMAT(ENND-L+1:ENND,ENND2-L+1:ENND2)=TEMPF1
	  ENDIF
	  IF((I-1.EQ.J).AND.(I.EQ.3)) THEN
        ENND=ENND+N_VAR_TOT
	    ENND2=ENND2+N_VAR_TOT
	    FMAT(ENND-N_VAR_TOT+1:ENND,ENND2-N_VAR_TOT+1:ENND2)=TEMPF2
	  ENDIF	    
	  IF((I-1.EQ.J).AND.(I.EQ.4)) THEN
        ENND=ENND+N_VAR_TOT
	    ENND2=ENND2+N_VAR_TOT+(M+R)*C
	    FMAT(ENND-N_VAR_TOT+1:ENND,ENND2-N_VAR_TOT+1:ENND2)=TEMPF2
	  ENDIF	    
	  IF((I-1.EQ.J).AND.(I.GT.4)) THEN
	    ENND=ENND+N_VAR_TOT
        ENND2=ENND2+N_VAR_TOT
	    FMAT(ENND-N_VAR_TOT+1:ENND,ENND2-N_VAR_TOT+1:ENND2)=TEMPF2
	  ENDIF
   END DO
END DO   	    



write(11,*) "Number of lags : ", Q
WRITE(11,*) "INITIAL VALUES : X= ",X(1),", SIGMA= ",X(NUM_A+1)

t0 = cpsec()
call comp_lik(n,x,f)
t1=cpsec()
t10=t1-t0
write(11,*) ' time =  ', t1



XINI=X

XSCALE=1.0
FSCALE=1.0

IBTYPE=0
DO I=1,NUM_A
   XLB(I)=-10000.987D0
   XUB(I)=+10000.987D0
END DO


DO I=NUM_A+1,NUM_TOT
   XLB(I)=-10000.001
   XUB(I)=+10000.987D0
END DO




!IPARAM=0.0 

CALL DU4INF(IPARAM,RPARAM)
IPARAM(3)=99000
IPARAM(4)=99000
IPARAM(5)=99000

!RPARAM(2)=3.66D-7
!RPARAM(6)=2.5d0
RPARAM(6)=1.0300d0
!print*,"RR",rparam(6)
RPARAM(2)=5.0d-60

write(11,*) "step: ",rparam(2)
!write(11,*) "par1: ",rparam(1)
write(11,*) "maxstep: ",rparam(6)

CALL DB2ONF(OBJFUN,N,XINI,IBTYPE,XLB,XUB,XSCALE,FSCALE,IPARAM,RPARAM,X,F,WK,IWK)




JJ=NUM_TOT*3+1
DO I=1,NUM_TOT
   GRADX(I)=WK(JJ)
   JJ=JJ+1
END DO      

WRITE(11,*) "GRADIENT"
WRITE(11,*) GRADX



DU=0.0
XOLD=X
CRIT0=CRIT2


DO I=1,NUM_TOT
   XOLD(I)=X(I)
   EPS=0.0001*ABS(X(I))
   X(I)=X(I)+EPS
   CALL COMP_LIK(N,X,F)
   DU(:,I)=(CRIT2-CRIT0)/(EPS)
   X(I)=XOLD(I)
END DO


VAR_X2=MATMUL(TRANSPOSE(DU),DU)




print*,"computing var_x2...."

CALL DLINRG(NUM_TOT,VAR_X2,NUM_TOT,VAR_X2,NUM_TOT)



DO I=1,NUM_TOT
   DO J=1,NUM_TOT
      IF(I.EQ.J) STDV2(I)=VAR_X2(I,I)
   END DO
END DO

DO I=1,NUM_TOT
  IF (STDV2(I).GE.0.0D0) STDV2(I)=DSQRT(STDV2(I))
END DO

DO I=1,NUM_TOT
   T_STAT2(I)=X(I)/STDV2(I)
END DO

DO I=1,NUM_TOT
   RES_FIN(1,I)=X(I)
   RES_FIN(2,I)=STDV2(I)
   RES_FIN(3,I)=T_STAT2(I)
END DO



WRITE(11,*) "THE SOLUTION IS:"
WRITE(11,*) X
WRITE(11,*) "F= ", F
WRITE(11,*) "NUMBER OF ITERATIONS: ", IPARAM(4)

WRITE(11,*) " COEFFICIENT                STANDARD DEVIATION                 T-STAT2"
WRITE(11,*) RES_FIN


CALL STD_EXC(X,VAR_X2,EXC,EXC_STD)

WRITE(11,*) "EXC AND STANDARD ERROR: ", EXC,EXC_STD


1010 continue

CLOSE(7)
CLOSE(8)
CLOSE(10)
CLOSE(11)

END PROGRAM MAXLIK


SUBROUTINE OBJFUN(N,X,F)
  USE FIRST
  IMPLICIT NONE
  INTEGER(4) :: N
  REAL(8) :: F,X(N)
  ICALL=ICALL+1
  CALL COMP_LIK(N,X,F)
  WRITE(8,*) "F: ",F,"ICALL: ",ICALL
  WRITE(8,*) X
  REWIND(8)
8888 FORMAT(E16.6/E16.6/E16.6/E16.6/E16.6)
END SUBROUTINE OBJFUN

SUBROUTINE COMP_LIK(N,X,CRIT)
   

   USE FIRST
   IMPLICIT NONE
   INTEGER:: N
   REAL(LONG),INTENT(OUT) :: CRIT
   INTEGER(4) :: I,II,III,IS,J,JJ,JJJ,K,KK,KKK,KK2,INI,ENND,INI2,ENND2,ITRING
   REAL(LONG) :: X(N),PROD(N_MOM)
   REAL(LONG) :: COEF2(P,PQ1),COEF(P,P,(Q+1)),A(NUM_A),SIGMA(NUM_S),VARV(LQ1,LQ1)
   REAL(LONG) :: SIG(C,C,(M+R)),SIGTEMP((C*(M+R)),C),SIGT1(C,C)
   REAL(LONG) :: SIG1((C*(M+R)),C),SIG2(H)
   REAL(LONG) :: AA(N_VAR_TOT,LQ1),EYEC(C,C),I_VEC(C)
   REAL(LONG) :: ATEMP1((M+R)*C,(M+R)*C),ATEMP2((M+R)*C)
   REAL(LONG) :: ETATEMP(N_VAR_TOT,(M+R)*C),ATEMP(N_VAR_TOT,N_VAR_TOT,Q+1)  
   REAL(LONG) :: TEMPC1(C,C),TEMPC((M+R)*C,(M+R)*C),TEMPC2(C)
   REAL(LONG) :: EPST(N_VAR_TOT),GT(N_VAR_TOT,N_VAR_TOT)
   REAL(LONG) :: KSITT0(LQ1),KSITT1(LQ1),PTT0(LQ1,LQ1),PTT1(LQ1,LQ1)
   REAL(LONG) :: FACMAT(N_VAR_TOT,N_VAR_TOT),DET1,DET2,DETGT
   REAL(LONG) :: TEMPCRIT2, TEMPCRIT1(N_VAR_TOT)
   REAL(LONG) :: MAT2(LQ1*LQ1),GINV(N_VAR_TOT,N_VAR_TOT),TEMPMAT(LQ1,N_VAR_TOT)
   REAL(LONG) :: KGAIN(LQ1,N_VAR_TOT),TEMPK(LQ1,LQ1)
   INTEGER(4) :: SHAPEP0(2),SHAPEM2(1),LQ1LQ1,IFOUR
   
CALL UMACH(-2,11)

      DO 11 IFOUR=1,NUM_A
         A(IFOUR)=X(IFOUR)
11    CONTINUE

!      WRITE(11,*) "A"
!	  WRITE(11,*) A

      I=1
      DO IFOUR=NUM_A+1,NUM_TOT
	     SIGMA(I)=X(IFOUR)
		 I=I+1
	  END DO	   



!      WRITE(11,*) "A"
!	  WRITE(11,*) A


!      WRITE(11,*) "SIGMA"
!	  WRITE(11,*) SIGMA



!                     MATRIX AA IN MEASUREMENT EQUATION

      KK=0
      DO 1 I=1,M
         DO 2 J=1,PQ1
            IF (J.EQ.((I-1)*(Q+1)+1)) THEN
              COEF2(I,J)=1
            ELSE
			  IF (CONSTRAINT(I,J).EQ.1) THEN
			     COEF2(I,J)=0.0D0
              ELSEIF(CONSTRAINT(I,J).EQ.0) THEN
                 KK=KK+1
                 COEF2(I,J)=A(KK)
              ENDIF
            ENDIF
2        CONTINUE
1     CONTINUE



      DO 3 I=(M+1),(M+R)
         DO 4 J=(M*(Q+1)+1),PQ1
            IF (J .EQ. ((I-1)*(Q+1)+1)) THEN
               COEF2(I,J)=1
            ELSE
			IF (CONSTRAINT(I,J).EQ.1) THEN
			     COEF2(I,J)=0.0D0
              ELSEIF(CONSTRAINT(I,J).EQ.0) THEN
               KK=KK+1
               COEF2(I,J)=A(KK)
             ENDIF
            ENDIF
4        CONTINUE
3     CONTINUE

if(exc_sens.eq.1) then
	coef2(2,2)=A(kk+1)
	coef2(2,3)=A(kk+2)
endif

	
      IF (H.GT.0) THEN
         DO 5 I=(M+R+1),P
            DO 6 J=((M+R)*(Q+1)+1),PQ1
               IF (J.EQ.((I-1)*(Q+1)+1)) THEN
                  COEF2(I,J)=1
               ELSE
			      IF (CONSTRAINT(I,J).EQ.1) THEN
			         COEF2(I,J)=0.0D0
                  ELSEIF(CONSTRAINT(I,J).EQ.0) THEN
                     KK=KK+1
                     COEF2(I,J)=A(KK)
				  ENDIF
               ENDIF
6           CONTINUE
5        CONTINUE
      ENDIF



      DO 111 I=1,P
        KK=1   
        DO 222 J=1,P
           DO 333 K=1,(Q+1)
              COEF(I,J,K)=COEF2(I,KK)
              KK=KK+1
333        CONTINUE
222     CONTINUE
111   CONTINUE  


      EYEC=0.0D01
      DO I=1,C
		   EYEC(I,I)=1.0D0
      END DO
       
!	  WRITE(11,*) "EYEC"
!	  WRITE(11,*) EYEC       
	  I_VEC=1.0D0 

	  ATEMP=0.0D0
	  ATEMP1=0.0D0	 

      
      DO I=1,Q+1
	      ATEMP1=0.0D0
	     CALL KRONECKER(COEF(1:M+R,1:M+R,I),M+R,M+R,EYEC,C,C,ATEMP1)
		 ATEMP(1:(M+R)*C,1:(M+R)*C,I)=ATEMP1
		 DO J=1,H
		    ATEMP2=0.0D0
		    CALL KRONECKER(COEF(1:M+R,M+R+J,I),M+R,1,I_VEC,C,1,ATEMP2)		
		    ATEMP(1:(M+R)*C,(M+R)*C+J,I)=ATEMP2
		    ATEMP((M+R)*C+1:N_VAR_TOT,(M+R)*C+J,I)=COEF(M+R+1:P,M+R+J,I)	
		 END DO 	 
       END DO
      


	   ETATEMP=0.0D0
	   DO I=1,C*(M+R)
	      ETATEMP(I,I)=1.0D0		
       END DO

INI=1
ENND=N_VAR_TOT
      DO I=1,Q+1
	     AA(:,INI:ENND)=ATEMP(:,:,I)
		 INI=ENND+1
		 ENND=ENND+N_VAR_TOT
		 IF(I.EQ.1) THEN 
		 AA(:,INI:ENND-H)=ETATEMP
         INI=ENND-H+1
		 ENND=ENND+(M+R)*C  
		 ELSEIF(I.EQ.2) THEN
         AA(:,INI:ENND-H)=-ETATEMP
         INI=ENND-H+1
		 ENND=ENND+(M+R)*C 
		 ENDIF
	  END DO	  


ITRING=0
!CALL DWRRRN("coef2",p,pQ1,coef2,p,ITRING)

!CALL DWRRRN("AA",N_VAR_TOT,LQ1,AA,N_VAR_TOT,ITRING)

!ITRING=0

!CALL DWRRRN("fmat",lq1,LQ1,fmat,lq1,ITRING)

!   *******                Reshape sigma parameters                   ******

      JJ=1
      JJJ=1
      DO 7 J=1,(M+R)
         DO 8 II=1,C
            DO 9 III=1,C
               IF(II.GT.III) THEN
                  SIG1(JJJ,III)=0
               ELSE
                  SIG1(JJJ,III)=SIGMA(JJ)
                  JJ=JJ+1
               ENDIF
9           CONTINUE
            JJJ=JJJ+1
8        CONTINUE
7     CONTINUE


      INI=1
      DO 15 I=1,(M+R)
         SIGT1=SIG1(INI:INI+C,:)
         SIG(:,:,I)=MATMUL(TRANSPOSE(SIGT1),SIGT1)
         INI=INI+C
15    CONTINUE




      IF(H.GT.0)THEN
       J=1
        DO 19 I=(NUM_S-H+1),(NUM_S)
         SIG2(J)=SIGMA(I)*SIGMA(I)
         J=J+1
19      CONTINUE
      ENDIF

!  ******            RESHAPE CORRECTION MATRICES            *********

    INI=1
	ENND=C

    DO I=1,M+R
	   INI2=1
	   ENND2=C
	   DO J=1,M+R
	      TEMPC1=0.0D0
	      DO K=1,C
	         TEMPC1(K,K)=CORR_0(I,J,K)
		  END DO	   
		  TEMPC(INI:ENND,INI2:ENND2)=TEMPC1
	      INI2=ENND2+1
	      ENND2=ENND2+C
	   END DO
	   INI=ENND+1
	   ENND=ENND+C
	END DO    

!  ********                   COMPUTE VARV                 ***********

   VARV=0.0D0
   INI=1
   ENND=C
   DO I=1,M+R
      VARV(INI:ENND,INI:ENND)=SIG(:,:,I)
	  INI=INI+C
	  ENND=ENND+C
   END DO

   DO I=1,H
      	VARV(INI:INI,INI:INI)=SIG2(I)
		INI=INI+1
   END DO
   
   VARV(N_VAR_TOT+1:N_VAR_TOT+C*(M+R),N_VAR_TOT+1:N_VAR_TOT+C*(M+R))=TEMPC	  
      

!CALL DWRRRN("TEMPC",(M+R)*C,(M+R)*C,TEMPC,(M+R)*C,ITRING)
!CALL DWRRRN("VARV",LQ1,LQ1,VARV,LQ1,ITRING)





! **********                 KALMAN RECURSIONS              ***************


KSITT1=0.0D0


!PTT1=VARV

PTT1=0.0D0

CRIT=0.0D0



DO I=1,T-1

   PTT0=MATMUL(MATMUL(FMAT,PTT1),TRANSPOSE(FMAT))+VARV
   KSITT0=MATMUL(FMAT,KSITT1)


   EPST=XT(:,I)-MATMUL(AA,KSITT0)
   GT=MATMUL(MATMUL(AA,PTT0),TRANSPOSE(AA))

!CALL DWRRRN("GT",N_VAR_TOT,N_VAR_TOT,GT,N_VAR_TOT,ITRING)
   CALL DLFTDS(N_VAR_TOT,GT,N_VAR_TOT,FACMAT,N_VAR_TOT)
   CALL DLFDDS(N_VAR_TOT,FACMAT,N_VAR_TOT,DET1,DET2)       ! DETERMINANT
   DETGT=(DET1*(10**DET2))
  
    

   CALL DLINDS(N_VAR_TOT,GT,N_VAR_TOT,GINV,N_VAR_TOT)        ! INVERSE
   TEMPCRIT1=(MATMUL(GINV,EPST))  
   TEMPCRIT1=EPST*TEMPCRIT1      
   TEMPCRIT2=SUM(TEMPCRIT1)
      
   CRIT=CRIT+0.5*(DLOG(DETGT)+TEMPCRIT2)
   CRIT2(I)=+0.5*(DLOG(DETGT)+TEMPCRIT2)  
   

   KGAIN=MATMUL(MATMUL(PTT0,TRANSPOSE(AA)),GINV)
   KSITT1=KSITT0+MATMUL(KGAIN,EPST)   
   PTT1=PTT0-MATMUL(MATMUL(KGAIN,AA),PTT0)
   
END DO



!WRITE(11,*) "CRIT = ",CRIT

1111 FORMAT(3E16.6/3E16.6/3E16.6/3E16.6/3E16.6)
END SUBROUTINE COMP_LIK



!************************************************************************

SUBROUTINE COLLAPSE(YRQ,MCELL,MCTRUE,COH,Y,INTRT,XTOT,CORR0)

USE FIRST
IMPLICIT NONE

REAL(LONG),INTENT(IN):: Y(M+R,MAXCELL,C,T),INTRT(T,H)
INTEGER(4) :: MCELL(C,T),MCTRUE(C,T,M+R), COH(T_IND),YRQ(T_IND)
REAL(LONG) :: YSTD(M+R,M+R,C,T),MEANINT(H),CORR0(M+R,M+R,C)
REAL(LONG):: YMEAN(M+R,C,T),YY(M+R,C),YMD(M+R,C,T-1),XTOT(N_VAR_TOT,T-1)
INTEGER(4) :: I,II,J,JJ,JJJ,K,KK,KKK,LAG,INI,ENND,ITRING
REAL(LONG) :: XPRINT(T-1,N_VAR_TOT)
INTEGER(4) :: STDCELL(M+R,M+R,C,T)

CALL UMACH(-2,11)


! ******************           COLLAPSE                 ***************

! Collapse data by yrq and coh, data are already sorted and no missing

YMEAN=0.0
YSTD=0.0

!  compute means of variables in level YMEAN(P,C,T)

DO I=1,M+R
   DO J=1,T
      DO K=1,C
         DO KK=1,MCELL(K,J)		 
		    IF (INT(ANINT(Y(I,KK,K,J)),4).NE.9990) THEN
            YMEAN(I,K,J)=YMEAN(I,K,J)+(Y(I,KK,K,J)/MCTRUE(K,J,I))
			ENDIF
         END DO
      END DO
   END DO
END DO

!write(9,*) mctrue

! take first differences   YMD(P,C,T-1)

YMD=0.0

DO I=1,M+R
   DO J=1,C
      DO K=2,T
         YMD(I,J,K-1)=YMEAN(I,J,K)-YMEAN(I,J,K-1)
      END DO
   END DO
END DO

! compute mean by cohort of ymd YY(P,C)

YY=0.0
DO I=1,M+R
   DO J=1,C
      DO K=1,T-1
         YY(I,J)=YY(I,J)+YMD(I,J,K)/(T-1)
      END DO
   END DO
END DO

! compute ymd=ymd-yy (M+R,C,T-1)

DO I=1,M+R
   DO J=1,C
      DO K=1,T-1
         YMD(I,J,K)=YMD(I,J,K)-YY(I,J)
      END DO
   END DO
END DO


! COMPUTE V1(N_V1)

V1=0.0D0


JJ=0
DO I=1,M+R
   DO J=I,M+R
      JJ=JJ+1	 
      DO K=1,T-1	
		 V1(JJ)=V1(JJ)+YMD(I,1,K)*YMD(J,1,K)/(T-1)
	  END DO
   END DO
END DO

! COMPUTE V2(N_V1)

V2=0.0D0


JJ=0
DO I=1,M+R
   DO J=I,M+R
      JJ=JJ+1	 
      DO K=1,T-1	
		 V2(JJ)=V2(JJ)+YMD(I,2,K)*YMD(J,2,K)/(T-1)
	  END DO
   END DO
END DO

! REPSHAPE MATRIX TO CREATE XT((M+R)*C,T-1)

XTOT=0.0D0
DO K=1,T-1
   JJ=1
   DO I=1,M+R
      DO J=1,C
	     XTOT(JJ,K)=YMD(I,J,K)
		 JJ=JJ+1
      END DO
   END DO
END DO

! compute standard deviations YSTD(M+R,M+R,C,T)

STDCELL=0

DO I=1,M+R
   DO J=1,M+R
      DO K=1,T
         DO KK=1,C		    
            DO JJ=1,MCELL(KK,K)
IF ((INT(ANINT(Y(I,JJ,KK,K)),4).NE.9990).AND.(INT(ANINT(Y(J,JJ,KK,K)),4).NE.9990)) THEN
               YSTD(I,J,KK,K)=YSTD(I,J,KK,K)+((Y(I,JJ,KK,K)-YMEAN(I,KK,K))*  &
&                          (Y(J,JJ,KK,K)-YMEAN(J,KK,K)))
               STDCELL(I,J,KK,K)=STDCELL(I,J,KK,K)+1   
            ENDIF 
            END DO
			YSTD(I,J,KK,K)=YSTD(I,J,KK,K)/STDCELL(I,J,KK,K)
         END DO
      END DO
   END DO
END DO


!  COMPUTE CORRECTION MATRIX FOR T=0  CORR_0(M+R,M+R,C)

CORR_0=0.0

DO I=1,M+R
   DO II=1,M+R
      DO J=1,C
         DO K=2,T
          CORR0(I,II,J)=CORR0(I,II,J)+(YSTD(I,II,J,K)/STDCELL(I,II,J,K))/(T-1)
         END DO
      END DO
   END DO
END DO



! INTEREST RATE: PUT IT IN DEVIATIONS FROM MEAN

MEANINT=0.0

DO I=1,H
   DO K=1,T
      MEANINT(I)=MEANINT(I)+INTRT(K,I)/T
   END DO
END DO

! drop FIRST observation so sample is the same as m+r variables INRT(T-1,H)

J=1
DO I=(M+R)*C+1,N_VAR_TOT,1   
   DO K=2,T
      XTOT(I,K-1)=INTRT(K,J)-MEANINT(J)
   END DO
   J=J+1
END DO

XPRINT=TRANSPOSE(XTOT)

!ITRING=0
!CALL DWRRRN("XTOT",T-1,N_VAR_TOT,XPRINT,T-1,ITRING)


END SUBROUTINE COLLAPSE


SUBROUTINE KRONECKER(A,RA,CA,B,RB,CB,CMAT)
USE FIRST
IMPLICIT NONE
INTEGER(4) :: RA,RB,CA,CB
INTEGER(4) :: INI,ENND,INI2,ENND2,I,J
REAL(LONG),INTENT(OUT) :: CMAT(RA*RB,CA*CB)
REAL(LONG) :: A(RA,CA),B(RB,CB)

INI=1
ENND=RB
DO I=1,RA
   INI2=1
   ENND2=CB
   DO J=1,CA
      CMAT(INI:ENND,INI2:ENND2)=A(I,J)*B	  
	  INI2=ENND2+1
	  ENND2=ENND2+CB
   END DO
   INI=ENND+1
   ENND=ENND+RB
END DO


END SUBROUTINE KRONECKER


SUBROUTINE STD_EXC(X,VAR_X,EXC,EXC_STD)
  USE FIRST
  IMPLICIT NONE
  REAL(8) :: X(NUM_TOT),VAR_X(NUM_TOT,NUM_TOT)
  REAL(8),INTENT(OUT) :: EXC,EXC_STD
  REAL(8) :: VAR_X2(NUM_TOT), SELE(NUM_TOT),SELE3(NUM_TOT),SELE4(NUM_TOT),INTEREST
  INTEGER(4) :: SELE1(NUM_TOT),SELE2(NUM_TOT),NCONSTR_ND,NCONSTR_Y,NCONSTR_R
  INTEGER :: I,J,K,KK,II,NUM_SELE


	IF(R.EQ.0) GO TO 14

	INTEREST=0.01

NCONSTR_ND=0
NCONSTR_Y=0
NCONSTR_R=0

	IF(H.GT.0) THEN
		DO I=1,PQ1-(1+Q)-(1+Q)
			NCONSTR_ND=NCONSTR_ND+CONSTRAINT(1,I)
		ENDDO
	
		DO I=PQ1-(1+Q)-(1+Q)+1,PQ1-(1+Q)
			NCONSTR_Y=NCONSTR_Y+CONSTRAINT(2,I)
		ENDDO

		DO I=PQ1-(1+Q)+1,PQ1
			NCONSTR_R=NCONSTR_R+CONSTRAINT(2,I)
		ENDDO
	ELSE

		DO I=1,PQ1-(1+Q)
			NCONSTR_ND=NCONSTR_ND+CONSTRAINT(1,I)
		ENDDO
	
		DO I=PQ1-(1+Q)+1,PQ1
			NCONSTR_Y=NCONSTR_Y+CONSTRAINT(2,I)
		ENDDO
ENDIF


	SELE1=0

	IF(NDY.EQ.1) THEN
		K=M*Q-NCONSTR_ND+1
		SELE1(K:K+Q)=1
		SELE2=0
		IF(H.NE.0) THEN
		SELE2(NUM_TOT-NUM_S-Q+1-(Q+1-NCONSTR_R)-(Q-NCONSTR_Y):NUM_TOT-NUM_S-Q+1-(Q+1-NCONSTR_R)-1)=1
		ELSEIF(H.EQ.0) THEN
		SELE2(NUM_TOT-NUM_S+1-(Q-NCONSTR_Y):NUM_TOT-NUM_S)=1
		ENDIF
		SELE=0
		SELE=SELE1-SELE2
	
	ELSE
		SELE1(M*Q-NCONSTR_ND+1:M*Q-NCONSTR_ND+Q+1)=1
!		SELE2=0
		SELE2(NUM_TOT-NUM_S-(Q-NCONSTR_Y)+1:NUM_TOT-NUM_S)=1
	ENDIF
	
	
	J=0	
	DO I=1,NUM_TOT
		IF(SELE2(I).EQ.1) THEN
!		IF(NDY.EQ.1) THEN
			J=J+1
			SELE4(I)=((1/(1+interest))**J)
!		ELSE
!		SELE4(I)=1.0D0
!		ENDIF
		ENDIF
	ENDDO

	
	J=0
	DO I=1,NUM_TOT
		IF(SELE1(I).EQ.1) THEN
		IF(NDY.EQ.0) THEN
		SELE3(I)=(1/(1+interest))**J
		J=J+1
		ELSE
		SELE3(I)=1.0D0
		ENDIF
		ENDIF
	ENDDO
	

	SELE=0
	SELE=SELE3-SELE4
	
	
PRINT*,"EXC"
PRINT*,"NCONSTR_ND",NCONSTR_ND
PRINT*,"NCONSTR_Y",NCONSTR_Y
PRINT*,"NCONSTR_R",NCONSTR_R

PRINT*,SELE

IF(NDY.EQ.1) THEN
	EXC=DOT_PRODUCT(SELE3,X)-(1+(DOT_PRODUCT(SELE4,X)))
ELSE
	EXC=(1+dot_product(sele4,x))-(DOT_PRODUCT(SELE3,X))
ENDIF


	DO I=1,NUM_TOT
		VAR_X2(I)=DOT_PRODUCT(SELE,VAR_X(:,I))		
	ENDDO

	EXC_STD=DOT_PRODUCT(VAR_X2,SELE)

	EXC_STD=SQRT(EXC_STD)


PRINT*,EXC

14 CONTINUE
    
  
END SUBROUTINE STD_EXC

