/*** drop if...  sample selection  ***/
clear
set mem 250M
use fes7405
gen yob=year-agehd


gen coh=1 if yob>=1940 & yob<1947
replace coh=2 if yob>=1947 & yob<1954

drop if coh==.

drop if empstat==2  /* drop self-employed */

keep if marstat<=3     /*** married or cohabitating ***/

gen byte quart=1 if month>=1 & month<4
replace quart=2 if month>=4 & month<7
replace quart=3 if month>=7 & month<10
replace quart=4 if month>=10 & month<=12

gen yrq=year*100+quart

gen yob2=1943 if coh==1
replace yob2=1950 if coh==2



gen byte qb=0
replace qb=.25 if quart==2
replace qb=.50 if quart==3
replace qb=.75 if quart==4

gen age=(year+qb)-yob2
*replace age=(age-40)/10

gen income1=gross_earnings
gen income2=gross_earnings+benefits
gen income3=net_earnings+benefits

gen ndur=nondur+services
gen totc=nondur+services+semidur+durables

replace income1=(income1/rpi)*100
replace income2=(income2/rpi)*100
replace income3=(income3/rpi)*100

replace ndur=(ndur/rpi)*100
replace totc=(totc/rpi)*100

keep if income1>0 & income3>0   

count

/* trim data */

sort yrq coh
egen minval=pctile(ndur),p(1) by(yrq coh)
sort coh
qui by coh: replace ndur=-888 if ndur<=minval 

sort yrq coh
egen maxval=pctile(ndur), p(99) by(yrq coh)
sort yrq coh
qui by yrq coh: replace ndur=-888 if ndur>=maxval & ndur!=. 

drop minval maxval
drop if int(ndur)==-888



sort yrq coh
egen minval=pctile(income1),p(1) by(yrq coh)
sort yrq coh
qui by yrq coh: replace income1=-888 if income1<=minval

sort yrq coh
egen maxval=pctile(income1), p(99) by(yrq coh)
sort yrq coh
qui by yrq coh: replace income1=-888 if income1>=maxval & ndur!=.

drop minval maxval
drop if int(income1)==-888

count

drop if income1>300
count


ta quart,gen(q)

drop agehd

gen age2=age*age/10


/**** regressions in levels ****/

reg ndur age* q2-q4 ahce if coh==1
predict e,res, if coh==1
replace ndur=e if coh==1
drop e 

reg ndur age* q2-q4 ahce if coh==2
predict e,res, if coh==2
replace ndur=e if coh==2
drop e 

reg totc age* q2-q4 ahce if coh==1
predict e,res, if coh==1
replace totc=e if coh==1
drop e 

reg totc age* q2-q4 ahce if coh==2
predict e,res, if coh==2
replace totc=e if coh==2
drop e 

reg income1 age* q2-q4 ahce if coh==1
predict e,res, if coh==1
replace income1=e if coh==1
drop e 

reg income1 age* q2-q4 ahce if coh==2
predict e,res, if coh==2
replace income1=e if coh==2
drop e 

reg income2 age* q2-q4 ahce if coh==1
predict e,res, if coh==1
replace income2=e if coh==1
drop e 

reg income2 age* q2-q4 ahce if coh==2
predict e,res, if coh==2
replace income2=e if coh==2
drop e 

reg income3 age* q2-q4 ahce if coh==1
predict e,res, if coh==1
replace income3=e if coh==1
drop e 

reg income3 age* q2-q4 ahce if coh==2
predict e,res, if coh==2
replace income3=e if coh==2
drop e 

keep income* ndur totc yrq coh
sort yrq coh


by yrq coh: gen cell=_N
qui by yrq coh: gen celly=_N

order yrq coh celly cell ndur totc income1 income2 income3 
sort yrq coh

save temp_lev_2coh,replace



/*** outfile data to be used as input in Fotran files ***/


u temp_lev_2coh,clear

keep if coh==1 | coh==2
count
collapse celly cell,by(yrq coh)
keep celly cell                            /** note in this case cell and celly are the same **/
su

outfile using cellnd&y_2o.dat,replace




u temp_lev_2coh,clear

keep yrq coh cell ndur income1
order yrq coh cell income ndur
outfile using y&nd1_2o.dat,replace

u temp_lev_2coh,clear
keep yrq coh cell ndur income2
order yrq coh cell income ndur
outfile using y&nd2_2o.dat,replace

u temp_lev_2coh,clear
keep yrq coh cell ndur income3
order yrq coh cell income ndur
outfile using y&nd3_2o.dat,replace

u temp_lev_2coh,clear
keep yrq coh cell totc income1
order yrq coh cell income totc
outfile using y&tc1_2o.dat,replace

u temp_lev_2coh,clear
keep yrq coh cell totc income2
order yrq coh cell income totc
outfile using y&tc2_2o.dat,replace

u temp_lev_2coh,clear
keep yrq coh cell totc income3
order yrq coh cell income totc
outfile using y&tc3_2o.dat,replace

