/*** drop if...  sample selection  ***/
clear
set mem 250M
use fes7405

gen yob=year-agehd

gen coh=1 if yob>=1940 & yob<1947
replace coh=2 if yob>=1947 & yob<1954

drop if coh==.

drop if empstat==2  /* drop self-employed */

keep if marstat<=3   /* keep if married or cohabitating */


gen byte quart=1 if month>=1 & month<4
replace quart=2 if month>=4 & month<7
replace quart=3 if month>=7 & month<10
replace quart=4 if month>=10 & month<=12

gen yrq=year*100+quart


gen yob2=1943 if coh==1
replace yob2=1950 if coh==2

gen byte qb=0
replace qb=.25 if quart==2
replace qb=.50 if quart==3
replace qb=.75 if quart==4

gen age=(year+qb)-yob2

gen income1=gross_earnings
gen income2=gross_earnings+benefits
gen income3=net_earnings+benefits

gen ndur=nondur+services
gen totc=nondur+services+semidur+durables

replace income1=(income1/rpi)*100
replace income2=(income2/rpi)*100
replace income3=(income3/rpi)*100

replace ndur=(ndur/rpi)*100
replace totc=(totc/rpi)*100

gen l_income1=log(income1)
gen l_income2=log(income2)
gen l_income3=log(income3)
gen l_ndur=log(ndur)
gen l_totc=log(totc)

gen l_ahce=log(ahceqscale)


keep if income1>0 & income3>0   /**** drop zero gross earnings!! + 1 negative income3 ****/

/** trim data **/

sort coh
egen minval=pctile(ndur),p(1) by(yrq coh)
sort coh
qui by coh: replace ndur=-888 if ndur<=minval

sort yrq coh
egen maxval=pctile(ndur), p(99) by(yrq coh)
sort yrq coh
qui by yrq coh: replace ndur=-888 if ndur>=maxval & ndur!=.

drop minval maxval
drop if int(ndur)==-888


sort yrq coh
egen minval=pctile(income1),p(4) by(yrq coh)
sort yrq coh
qui by yrq coh: replace income1=-888 if income1<=minval

sort yrq coh
egen maxval=pctile(income1), p(96) by(yrq coh)
sort yrq coh
qui by yrq coh: replace income1=-888 if income1>=maxval & totc!=.

drop minval maxval
drop if int(income1)==-888
drop if income1>300


gen age2=age*age/10
gen age3=age2*age/100
gen age4=age3*age/1000
gen age5=age4*age/10000

ta quart,gen(q)

drop agehd




/**** regressions in logs ****/

reg l_ndur age* q2-q4 l_ahce if coh==1 
predict e,res, if e(sample)
replace l_ndur=e if coh==1
drop e 

reg l_ndur age* q2-q4 l_ahce if coh==2 
predict e,res, if e(sample)
replace l_ndur=e if coh==2
drop e 

reg l_totc age* q2-q4 l_ahce if coh==1
predict e,res, if coh==1
replace l_totc=e if coh==1
drop e 

reg l_totc age* q2-q4 l_ahce if coh==2
predict e,res, if coh==2
replace l_totc=e if coh==2
drop e 

reg l_income1 age* q2-q4 l_ahce if coh==1
predict e,res, if coh==1
replace l_income1=e if coh==1
drop e 

reg l_income1 age* q2-q4 l_ahce if coh==2
predict e,res, if coh==2
replace l_income1=e if coh==2
drop e 

reg l_income2 age* q2-q4 l_ahce if coh==1
predict e,res, if coh==1
replace l_income2=e if coh==1
drop e 

reg l_income2 age* q2-q4 l_ahce if coh==2
predict e,res, if coh==2
replace l_income2=e if coh==2
drop e 

reg l_income3 age* q2-q4 l_ahce if coh==1
predict e,res, if coh==1
replace l_income3=e if coh==1
drop e 

reg l_income3 age* q2-q4 l_ahce if coh==2
predict e,res, if coh==2
replace l_income3=e if coh==2
drop e 

keep l_income* l_ndur l_totc yrq coh
sort yrq coh
qui by yrq coh: gen cell=_N
qui by yrq coh: gen celly=_N

order yrq coh celly cell l_ndur l_totc l_income1 l_income2 l_income3
sort yrq coh
save temp_log_2coh,replace

/*** outfile data to be used as input in Fotran files ***/

collapse cell celly,by(yrq coh)
keep celly cell                                   /** note in this case cell and celly are the same **/

su
outfile using celltc&y_2o.dat,replace



u temp_log_2coh,clear
keep yrq coh cell l_ndur l_income1
order yrq coh cell l_income l_ndur
outfile using logy&nd1_2o.dat,replace

u temp_log_2coh,clear
keep yrq coh cell l_ndur l_income2
order yrq coh cell l_income l_ndur
outfile using logy&nd2_2o.dat,replace

u temp_log_2coh,clear
keep yrq coh cell l_ndur l_income3
order yrq coh cell l_income l_ndur
outfile using logy&nd3_2o.dat,replace

u temp_log_2coh,clear
keep yrq coh cell l_totc l_income1
order yrq coh cell l_income l_totc
outfile using logy&tc1_2o.dat,replace

u temp_log_2coh,clear
keep yrq coh cell l_totc l_income2
order yrq coh cell l_income l_totc
outfile using logy&tc2_2o.dat,replace

u temp_log_2coh,clear
keep yrq coh cell l_totc l_income3
order yrq coh cell l_income l_totc
outfile using logy&tc3_2o.dat,replace

